//
// QtUPDDPaintDemo
//
// TBReceiver.cpp
// Copyright 2013 Touch-Base Ltd. All Rights Reserved
//

#include <set>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <QDebug>
#include <QFile>
#include <QApplication>

#ifndef Q_WS_WIN
#include "nonwindows.h"
#endif
#include "tbapi.h"
#include "TBReceiver.h"
#include "logWindow.h"

using namespace std;
QPaintWidget *paintWidget;

void _stdcall TBCallback(unsigned long, _PointerData* data)
{
    static map<int,bool> touching;
	static map<int, QPointF > coordinate;
	int stylusIndex = data->hStylus;

	if(touching.find(stylusIndex) == touching.end()) {
		touching[stylusIndex]=false;
    }

	if (data->type == _ReadDataTypeEvent) {
        if(touching[stylusIndex] != bool(data->pd.event.left)) {
			assert(coordinate.find(stylusIndex) != coordinate.end()); // should never happen

			if(data->pd.event.left) {
				// touchdown
				paintWidget->addTouch(stylusIndex, coordinate[stylusIndex]);
			} else {
				// liftoff
				paintWidget->removeTouch(stylusIndex);
            }
			touching[stylusIndex] = data->pd.event.left;
        }
	}
	if(data->type == _ReadDataTypeXY) {
        double x = double(data->pd.xy.calx) / 65535.0;
        double y = double(data->pd.xy.caly) / 65535.0;
		coordinate[stylusIndex] = QPointF(x, y);

		if(touching[stylusIndex]) {
			paintWidget->updateTouch(stylusIndex, coordinate[stylusIndex]);
        }
    }

	if(data->type == _ReadDataTypeUnload) {
        TBTerminateProcess();
    }
}

bool TBInitialize(QPaintWidget *widget)
{
	
	char buffer[1024];
	
	TBApiDefaultSettingsPath(buffer, 1024);
#ifdef Q_WS_MAC
	// TBApiDefaultSettingsPath sometimes returns the old location of tbupdd.ini. Working
	// around it by checking for its existence in the reported directory and using the
	// new default path if it's not found
	QString iniPath = QString(buffer) + "tbupdd.ini";
	if (!QFile::exists(iniPath)) {
		strcpy(buffer, "/Library/Preferences/");
	}
#endif
	TBApiInit(buffer);

	if(!TBApiOpen()) {
        qDebug() << "Critical error: TBApiOpen failure.  Cannot receive touch events from the UPDD.";
		return(false); 
	}
	
	TBApiMousePortInterfaceEnable(false);
	
	TBApiRegisterDataCallback(	0,  // specify 0 to receive callbacks for all active devices or a handle for a specific device 
								0,  // this value gets passed to the callback function 
								_ReadDataTypeUnload |     // notification that we must unload (during driver uninstall) 
								_ReadDataTypeEvent |      // a mouse button state change is indicated by the pointer device
								_ReadDataTypeXY,          // a mouse position change is indicated by the pointer device
								TBCallback);
    
	paintWidget = widget;

	return true;
}


void TBDeinitialize()
{
	TBApiUnregisterDataCallback(TBCallback);
	TBApiMousePortInterfaceEnable(true);
	TBApiClose();
    TBApiTerminate();
}

void TBTerminateProcess()
{
	qApp->quit();
}


