//
// QtUPDDPaintDemo
//
// logwindow.cpp
// Copyright 2013 Touch-Base Ltd. All Rights Reserved
//

#include <iostream>

#include <QString>
#include <QApplication>
#include <QBuffer>
#include <QDebug>
#include <QMutex>
#include <QPushButton>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QScrollBar>

#include "logwindow.h"

QMutex logMutex;

LogWindow *logWindow = NULL;
QTextEdit *logTextEdit = NULL;

LogWindow::LogWindow(QWidget *parent) : QMainWindow(parent)
{
	QWidget *centralWidget = new QWidget();
	QVBoxLayout *layout = new QVBoxLayout();
	QHBoxLayout *buttonLayout = new QHBoxLayout();
	QTextEdit *textEdit = new QTextEdit();
	QPushButton *clearButton = new QPushButton("Clear Log");
	
	QFont font("Monospace");
	font.setPointSize(font.pointSize()-1);
	font.setBold(true);
	font.setStyleHint(QFont::TypeWriter);
	
	textEdit->setFont(font);
	textEdit->setReadOnly(true);
	
	buttonLayout->addWidget(clearButton);
	buttonLayout->addStretch(0);
	
	layout->setMargin(2);
	layout->setSpacing(2);
	layout->addWidget(textEdit);
	layout->addLayout(buttonLayout);
	centralWidget->setLayout(layout);
	
	setCentralWidget(centralWidget);
	resize(640, 480);
	setWindowTitle("UPDD Touch Log");
	
	logTextEdit = textEdit;
	
	connect(clearButton, SIGNAL(clicked()), this, SLOT(clearLog()));
}

LogWindow::~LogWindow()
{
	logWindow = NULL;
	logTextEdit = NULL;
}

void LogWindow::clearLog()
{
	if(!logMutex.tryLock())
        return;
	
	logTextEdit->clear();
	
	logMutex.unlock();
}

void displayInLogWindow(QString message, QColor color)
{
	if(!logMutex.tryLock())
        return;
	
	if (logTextEdit) {
		logTextEdit->setTextColor(color);
		logTextEdit->append(message);
		//logTextEdit->update();

		// Scroll text to edit:
		QTextCursor c =  logTextEdit->textCursor();
		c.movePosition(QTextCursor::End);
		logTextEdit->setTextCursor(c);
	}
	
	logMutex.unlock();
}

void showLogWindow(QWidget *parent)
{	
	if(!logMutex.tryLock())
        return;
	
	if (!logWindow) {
		logWindow = new LogWindow(parent);
	}
	logWindow->show();
	logWindow->raise();
	
	logMutex.unlock();
}
