//
// QtUPDDPaintDemo
//
// utilities.cpp
// Copyright 2013 Touch-Base Ltd. All Rights Reserved
//

#include "utilities.h"

int min(int a, int b) {
    return (a < b) ? a : b;
}

int max(int a, int b) {
    return (a > b) ? a : b;
}

// Takes a sequence of data, the latest piece of which is defined by 'value', and smooths it by
// averaging it with previous values in the sequence.
// dataBuffer maintains a list of values from previous calls to this function.  Every call to this
// function should pass in the same QList for a particular set of data. 'valuesToSmooth'
// indicates how many previous pieces of data should be maintained.
// So if this function were called 5 times with the following values in order:
// 12.0, 10.0, 7.0, 8.0, 9.5
// and valuesToSmooth was 4, on return dataBuffer would contain 10, 7, 8, and 9.5 in that order,
// and the returned value would be 8.625
double smoothData(QList<double> &dataBuffer, int valuesToSmooth, double value)
{
    dataBuffer.append(value);
    if (dataBuffer.count() > valuesToSmooth) {
        dataBuffer.removeFirst();
    }
    double average = 0;
    for(int i = 0; i < dataBuffer.count(); ++i) {
        average += dataBuffer[i];
    }
    return average / dataBuffer.count();
}

// Returns the index of the button corresponding to the Qt flag passed in
// For our uses this function will only ever be passed in a value containing one flag, but in case there are multiple flags,
// it will return the button with the largest index
int buttonIndex(int buttonFlag)
{
    int index = 0;
    for(int i = 0; i < 32; ++i) { // There won't be more than 32 mouse buttons, right?  ....right?!
        if ((buttonFlag & (0x01 << i)) != 0)
            index = i;
    }
    return index;
}

// Convenience function, performs bounds checking on colorList and returns black if index is out of bounds
QColor retrieveColor(const QList<QColor> &colorList, int index)
{
    if (index >= 0 && index < colorList.count())
        return colorList[index];
    else
        return QColor(0.0, 0.0, 0.0);
}
